/**
    @file mfg_dfu.c

    @brief Implement a DFU Interface example project for the FT51.

    @version 1.0.0

**/
/*
 * ============================================================================
 * History
 * =======
 *
 * (C) Copyright 2014, Future Technology Devices International Ltd.
 * ============================================================================
 *
 * This source code ("the Software") is provided by Future Technology Devices
 * International Limited ("FTDI") subject to the licence terms set out
 * http://www.ftdichip.com/FTSourceCodeLicenceTerms.htm ("the Licence Terms").
 * You must read the Licence Terms before downloading or using the Software.
 * By installing or using the Software you agree to the Licence Terms. If you
 * do not agree to the Licence Terms then do not download or use the Software.
 *
 * Without prejudice to the Licence Terms, here is a summary of some of the key
 * terms of the Licence Terms (and in the event of any conflict between this
 * summary and the Licence Terms then the text of the Licence Terms will
 * prevail).
 *
 * The Software is provided "as is".
 * There are no warranties (or similar) in relation to the quality of the
 * Software. You use it at your own risk.
 * The Software should not be used in, or for, any medical device, system or
 * appliance. There are exclusions of FTDI liability for certain types of loss
 * such as: special loss or damage; incidental loss or damage; indirect or
 * consequential loss or damage; loss of income; loss of business; loss of
 * profits; loss of revenue; loss of contracts; business interruption; loss of
 * the use of money or anticipated savings; loss of information; loss of
 * opportunity; loss of goodwill or reputation; and/or loss of, damage to or
 * corruption of data.
 * There is a monetary cap on FTDI's liability.
 * The Software may have subsequently been amended by another user and then
 * distributed by that other user ("Adapted Software").  If so that user may
 * have additional licence terms that apply to those amendments. However, FTDI
 * has no liability in relation to those amendments.
 * ============================================================================
 */
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include "ft51.h"
#include "ft51_general_config.h"
#include "ft51_interrupts.h"
#include "ft51_usb.h"
#include "ft51_usb_dfu.h"
#include "ft51_dfu.h"
#include "ft51_iomux.h"
#include "registers\ft51_iomux_registers.h"

/**
 @name USB and Hub Configuration
 @brief Indication of whether the USB Hub is enabled, how it is
        powered and the size of the control endpoint Max Packets.
        Set ENABLE_LEDS undefined will disable the status LED for
        active DFU operation. This is the default option as
        toggling LEDs on boards may have undefined consequences.
 */
//@{
// USB Hub Enabled - set to 1 for enabled and 0 for disabled
#ifndef USB_HUB_ENABLED
#define USB_HUB_ENABLED 0
#endif // USB_HUB_ENABLED
// USB Bus Powered - set to 1 for self-powered or 0 for bus-powered
#ifndef USB_SELF_POWERED
#define USB_SELF_POWERED 0
#endif // USB_SELF_POWERED
// USB Remote Wakeup - set to 1 for enabled and 0 for disabled
#ifndef USB_REMOTE_WAKEUP
#define USB_REMOTE_WAKEUP 0
#endif // USB_REMOTE_WAKEUP

#if USB_SELF_POWERED == 1
#if USB_REMOTE_WAKEUP == 1
#define USB_CONFIG_BMATTRIBUTES_VALUE (USB_CONFIG_BMATTRIBUTES_SELF_POWERED | USB_CONFIG_BMATTRIBUTES_REMOTE_WAKEUP | USB_CONFIG_BMATTRIBUTES_RESERVED_SET_TO_1)
#else // USB_REMOTE_WAKEUP
#define USB_CONFIG_BMATTRIBUTES_VALUE (USB_CONFIG_BMATTRIBUTES_SELF_POWERED | USB_CONFIG_BMATTRIBUTES_RESERVED_SET_TO_1)
#endif // USB_REMOTE_WAKEUP
#else // USB_SELF_POWERED
#if USB_REMOTE_WAKEUP == 1
#define USB_CONFIG_BMATTRIBUTES_VALUE (USB_CONFIG_BMATTRIBUTES_REMOTE_WAKEUP | USB_CONFIG_BMATTRIBUTES_RESERVED_SET_TO_1)
#else // USB_REMOTE_WAKEUP
#define USB_CONFIG_BMATTRIBUTES_VALUE USB_CONFIG_BMATTRIBUTES_RESERVED_SET_TO_1
#endif // USB_REMOTE_WAKEUP
#endif // USB_SELF_POWERED

// Enable (non-zero) or disable (zero) device detach-attach
// sequences in DFU firmware.
// This is used in the bmAttributes of the USB_dfu_functional_descriptor.
#ifndef DFU_DETACH_MODE
#define DFU_DETACH_MODE 1
#endif // DFU_DETACH_MODE

// Enable or disable activity LEDs on FT51A EVM boards
#undef ENABLE_LEDS

// USB Endpoint Zero packet size (both must match)
#define USB_CONTROL_EP_MAX_PACKET_SIZE 8
#define USB_CONTROL_EP_SIZE USB_EP_SIZE_8
//@}

/**
 @name Board Configuration
 @brief Mapping of pins on test board.
 @		On FT51A EVM switch SW1 maps to DIO_0
 */
//@{
// USB Remote Wakeup Pin
#ifndef PIN_REMOTE_WAKEUP
#define PIN_REMOTE_WAKEUP DIO_0
#endif // PIN_REMOTE_WAKEUP
//@}

/**
    @name Device Configuration Areas
    @brief Size and location of string descriptors and
        structure used to modify device and configuration
        descriptors.
 */
//@{
// String descriptors - allow a maximum of 256 bytes for this
#define STRING_DESCRIPTOR_LOCATION 0x80
#define STRING_DESCRIPTOR_ALLOCATION 0x100
// Locate immediately after string descriptions
#define DEVICE_CONFIG_LOCATION STRING_DESCRIPTOR_LOCATION + STRING_DESCRIPTOR_ALLOCATION
//@}

/**
 @name DFU_BLOCK_SIZE definition.
 @brief Number of bytes in block, sent in each DFU_DNLOAD request
 from the DFU update program on the host. This is simplified
 in that the meaning of a block is an arbitrary number of
 bytes. This is intentionally a multiple of the maximum
 packet size for the control endpoints.
 */
//@{
#define DFU_BLOCK_SIZE DFU_MAX_BLOCK_SIZE
//@}

/**
    @name DFU_USB_PID_RUNTIME configuration.
    @brief Run Time Product ID for manufacturing DFU program.
 */
//@{
#define DFU_USB_PID_RUNTIME 0x0fed

/**
    @name DFU_USB_PID_DFUMODE configuration.
    @brief FTDI predefined DFU Mode Product ID.
 */
//@{
#define DFU_USB_PID_DFUMODE 0x0fee
//@}

/**
    @name DFU_USB_INTERFACE configuration.
    @brief Run Time and DFU Mode Interface Numbers.
 */
//@{
#define DFU_USB_INTERFACE_RUNTIME 0
#define DFU_USB_INTERFACE_DFUMODE 0
//@}

/**
 @name WCID_VENDOR_REQUEST_CODE for WCID.
 @brief Unique vendor request code for WCID OS Vendor Extension validation.
*/
//@{
#define WCID_VENDOR_REQUEST_CODE	 0xF1
//@}

/**
    @def MICROSECONDS_TO_TIMER_TICKS
    @brief Macro to calculate the number of timer ticks required to
     	create a 1ms counter.
    	Scale is clock frequency in MHz (assume 48) / clock-cycles per instruction.
		16-bit timer counts up to 65536, so subtract duration from 65536.
 */
#define MICROSECONDS_TO_TIMER_TICKS(a) (65536 - ((a) * (48 / 12)))

/**
 	 @name string_descriptor
 	 @brief Table of USB String descriptors

 	 This is placed at a fixed location in the __code section allowing
 	 up-to 256 bytes of string descriptors to be defined. These can be
 	 modified or replaced by a utility or binary editor without
 	 requiring a recompilation of the firmware.
 	 They are placed at offset 0x100 and reserve 0x100 bytes.
 	 The data is not stored in __xdata section and is therefore
 	 regarded as const.
 */
__code __at (STRING_DESCRIPTOR_LOCATION)
    uint8_t string_descriptor[STRING_DESCRIPTOR_ALLOCATION] =
{
        // String 0 is actually a list of language IDs supported by the real strings.
        0x04, USB_DESCRIPTOR_TYPE_STRING,
        0x09, 0x04, // 0409 = English (US)

        // String 1 (Manufacturer): "FTDI"
        0x0a, USB_DESCRIPTOR_TYPE_STRING,
        'F', 0x00, 'T', 0x00, 'D', 0x00, 'I', 0x00,

        // String 2 (Product): "FT51 Run Time"
        0x1A, USB_DESCRIPTOR_TYPE_STRING,
        'F', 0x00, 'T', 0x00, '5', 0x00, '1', 0x00,
        ' ', 0x00, 'R', 0x00, 'u', 0x00, 'n', 0x00,
        't', 0x00, 'i', 0x00, 'm', 0x00, 'e', 0x00,

        // String 3 (Serial Number): "FT424242"
        0x12, USB_DESCRIPTOR_TYPE_STRING,
        'F', 0x00, 'T', 0x00, '4', 0x00, '2', 0x00,
        '4', 0x00, '2', 0x00, '4', 0x00, '2', 0x00,

        // String 4 (DFU Product Name): "FT51 DFU Mode"
        0x1C, USB_DESCRIPTOR_TYPE_STRING,
        'F', 0x00, 'T', 0x00, '5', 0x00, '1', 0x00,
        ' ', 0x00, 'D', 0x00, 'F', 0x00, 'U', 0x00,
        ' ', 0x00, 'M', 0x00, 'o', 0x00, 'd', 0x00,
        'e', 0x00,

        // String 5 (Interface Name): "DFU Interface"
        0x1c, USB_DESCRIPTOR_TYPE_STRING,
        'D', 0x00, 'F', 0x00, 'U', 0x00, ' ', 0x00,
        'I', 0x00, 'n', 0x00, 't', 0x00, 'e', 0x00,
        'r', 0x00, 'f', 0x00, 'a', 0x00, 'c', 0x00,
        'e', 0x00,

        // END OF STRINGS
        0x00
};

/**
     @struct device_config
     @brief Modifiable configuration data structure for firmware.

     This is placed at a fixed location in the __code section to provide
     the ability to modify certain configuration details.
     As it appears at a fixed location in the binary (or hex) files it
     can be changed with a binary editor or utility without requiring a
     recompilation of the firmware.
 */
//@{
struct device_config {
    /** Vendor ID. **/
    uint16_t idVendor;
    /** Index of manufacturer string descriptor. **/
    uint8_t  iManufacturer;
    /** Index of serial no. string descriptor. **/
    uint8_t  iSerialNumber;
    /** Index of serial no. interface descriptor. **/
    uint8_t  iInterface;
    /** Product ID for Run Time. **/
    uint16_t idProductRunTime;
    /** Index of product string descriptor for Run Time. **/
    uint8_t  iProductRunTime;
    /** Product ID for DFU Mode. **/
    uint16_t idProductDFUMode;
    /** Index of product string descriptor for Run Time. **/
    uint8_t  iProductDFUMode;
    /** pad to 16 bytes **/
    uint8_t  pad[5];
};
__code __at (DEVICE_CONFIG_LOCATION) struct device_config device_config =
{
        USB_VID_FTDI,   // idVendor LSB: 0x0403 (FTDI)
        0x01,           // iManufacturer
        0x03,           // iSerialNumber
        0x05,           // iInterface
        DFU_USB_PID_RUNTIME, // Runtime idProduct: 0x0fed
        0x02,           // Runtime iProduct
        DFU_USB_PID_DFUMODE, // DFU Mode idProduct: 0x0fee
        0x04,           // DFU Mode iProduct
};
//@}

/**
 @name wcid_string_descriptor
 @brief USB String descriptor for WCID identification.
 */
__code uint8_t wcid_string_descriptor[USB_MICROSOFT_WCID_STRING_LENGTH] = {
		USB_MICROSOFT_WCID_STRING(WCID_VENDOR_REQUEST_CODE)
};

/**
 	 @name device_descriptor_runtime
 	 @brief Device descriptor for Run Time mode.

 	 Fields marked with * are updated with information in device_config[]
 */
USB_device_descriptor device_descriptor_runtime =
{
        .bLength = 0x12,
        .bDescriptorType = 0x01,
        .bcdUSB = USB_BCD_VERSION_2_0,
        .bDeviceClass = USB_CLASS_DEVICE,
        .bDeviceSubClass = USB_SUBCLASS_DEVICE,
        .bDeviceProtocol = USB_PROTOCOL_DEVICE,
        .bMaxPacketSize0 = USB_CONTROL_EP_MAX_PACKET_SIZE,
        .idVendor = USB_VID_FTDI,   // 0x0403 (FTDI)
        .idProduct = DFU_USB_PID_RUNTIME, // 0x0fed
        .bcdDevice = 0x0101,
        .iManufacturer = 0x01,  // String 1
        .iProduct = 0x02,       // String 2
        .iSerialNumber = 0x03,  // String 3
        .bNumConfigurations = 0x01
};

/**
 	 @name config_descriptor_runtime
 	 @brief Config descriptor for Run Time mode.

 	 Fields marked with * are updated with information in device_config[]
 */
//@{
// Structure containing layout of configuration descriptor
struct config_descriptor_runtime
{
    USB_configuration_descriptor configuration;
    USB_interface_descriptor interface;
    USB_dfu_functional_descriptor functional;
};

struct config_descriptor_runtime config_descriptor_runtime =
{
        .configuration.bLength = 0x09,
        .configuration.bDescriptorType = USB_DESCRIPTOR_TYPE_CONFIGURATION,
        .configuration.wTotalLength = sizeof(struct config_descriptor_runtime),
        .configuration.bNumInterfaces = 0x01,
        .configuration.bConfigurationValue = 0x01,
        .configuration.iConfiguration = 0x00,
        .configuration.bmAttributes = USB_CONFIG_BMATTRIBUTES_VALUE,
        .configuration.bMaxPower = 0xFA, // 500 mA

        // ---- INTERFACE DESCRIPTOR for DFU Interface ----
        .interface.bLength = 0x09,
        .interface.bDescriptorType = USB_DESCRIPTOR_TYPE_INTERFACE,
        .interface.bInterfaceNumber = DFU_USB_INTERFACE_RUNTIME,
        .interface.bAlternateSetting = 0x00,
        .interface.bNumEndpoints = 0x00,
        .interface.bInterfaceClass = USB_CLASS_APPLICATION, // Application Specific Class
        .interface.bInterfaceSubClass = USB_SUBCLASS_DFU,   // Device Firmware Update
        .interface.bInterfaceProtocol = USB_PROTOCOL_DFU_RUNTIME, // Runtime Protocol
        .interface.iInterface = 0x05, // String 5

        // ---- FUNCTIONAL DESCRIPTOR for DFU Interface ----
        .functional.bLength = 0x09,
        .functional.bDescriptorType = USB_DESCRIPTOR_TYPE_DFU_FUNCTIONAL,
#if DFU_DETACH_MODE
		.functional.bmAttributes = USB_DFU_BMATTRIBUTES_CANDNLOAD | USB_DFU_BMATTRIBUTES_WILLDETACH,
#else // DFU_DETACH_MODE
		.functional.bmAttributes = USB_DFU_BMATTRIBUTES_CANDNLOAD,
#endif // DFU_DETACH_MODE
        .functional.wDetatchTimeOut = DFU_TIMEOUT,
        .functional.wTransferSize = DFU_BLOCK_SIZE,
        .functional.bcdDfuVersion = USB_BCD_VERSION_DFU_1_1, // DFU Version 1.1
};
//@}

/**
 	 @name device_descriptor_dfumode
 	 @brief Device descriptor for DFU Mode.

 	 Fields marked with * are updated with information in device_config[]
 */
USB_device_descriptor device_descriptor_dfumode =
{
        .bLength = 0x12,
        .bDescriptorType = USB_DESCRIPTOR_TYPE_DEVICE,
        .bcdUSB = USB_BCD_VERSION_2_0,
        .bDeviceClass = USB_CLASS_DEVICE,
        .bDeviceSubClass = USB_SUBCLASS_DEVICE,
        .bDeviceProtocol = USB_PROTOCOL_DEVICE,
        .bMaxPacketSize0 = USB_CONTROL_EP_MAX_PACKET_SIZE,
        .idVendor = USB_VID_FTDI,   // 0x0403 (FTDI)
        .idProduct = DFU_USB_PID_DFUMODE, // 0x0fee
        .bcdDevice = 0x0101,
        .iManufacturer = 0x01,
        .iProduct = 0x04,
        .iSerialNumber = 0x03,
        .bNumConfigurations = 0x01
};

/**
 	 @name config_descriptor_dfumode
 	 @brief Config descriptor for DFU Mode.

 	 Fields marked with * are updated with information in device_config[]
 */
//@{
struct config_descriptor_dfumode
{
    USB_configuration_descriptor configuration;
    USB_interface_descriptor interface;
    USB_dfu_functional_descriptor functional;
};

struct config_descriptor_dfumode config_descriptor_dfumode =
{
        .configuration.bLength = 0x09,
        .configuration.bDescriptorType = USB_DESCRIPTOR_TYPE_CONFIGURATION,
        .configuration.wTotalLength = sizeof(struct config_descriptor_dfumode),
        .configuration.bNumInterfaces = 0x01,
        .configuration.bConfigurationValue = 0x01,
        .configuration.iConfiguration = 0x00,
        .configuration.bmAttributes = USB_CONFIG_BMATTRIBUTES_VALUE,
        .configuration.bMaxPower = 0xFA, // 500 mA

		// ---- INTERFACE DESCRIPTOR for DFU Interface ----
        .interface.bLength = 0x09,
        .interface.bDescriptorType = USB_DESCRIPTOR_TYPE_INTERFACE,
        .interface.bInterfaceNumber = DFU_USB_INTERFACE_DFUMODE,
        .interface.bAlternateSetting = 0x00,
        .interface.bNumEndpoints = 0x00,
        .interface.bInterfaceClass = USB_CLASS_APPLICATION, // Application Specific Class
        .interface.bInterfaceSubClass = USB_SUBCLASS_DFU,   // Device Firmware Update
        .interface.bInterfaceProtocol = USB_PROTOCOL_DFU_DFUMODE, // Runtime Protocol
        .interface.iInterface = 0x05, // String 5

        // ---- FUNCTIONAL DESCRIPTOR for DFU Interface ----
        .functional.bLength = 0x09,
        .functional.bDescriptorType = USB_DESCRIPTOR_TYPE_DFU_FUNCTIONAL,
#if DFU_DETACH_MODE
		.functional.bmAttributes = USB_DFU_BMATTRIBUTES_CANDNLOAD | USB_DFU_BMATTRIBUTES_WILLDETACH,
#else // DFU_DETACH_MODE
		.functional.bmAttributes = USB_DFU_BMATTRIBUTES_CANDNLOAD,
#endif // DFU_DETACH_MODE
        .functional.wDetatchTimeOut = DFU_TIMEOUT, // suggest 8192ms
        .functional.wTransferSize = DFU_BLOCK_SIZE,// typically 64 bytes
        .functional.bcdDfuVersion = USB_BCD_VERSION_DFU_1_1,
};
//@}

/**
 @name wcid_feature_runtime
 @brief WCID Compatible ID for DFU interface in runtime.
 */
//@{
__code USB_WCID_feature_descriptor wcid_feature_runtime =
{
	sizeof(struct USB_WCID_feature_descriptor), /* dwLength */
	USB_MICROSOFT_WCID_VERSION, /* bcdVersion */
	USB_MICROSOFT_WCID_FEATURE_WINDEX_COMPAT_ID, /* wIndex */
	1, /* bCount */
	{0, 0, 0, 0, 0, 0, 0,}, /* rsv1 */
	DFU_USB_INTERFACE_RUNTIME, /* bFirstInterfaceNumber */
	1, /* rsv2 - set to 1 */
	{'W', 'I', 'N', 'U', 'S', 'B', 0x00, 0x00,}, /* compatibleID[8] */
	{0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,}, /* subCompatibleID[8] */
	{0, 0, 0, 0, 0, 0,}, /* rsv3[6] */
};
//@}

/**
 @name wcid_feature_dfumode
 @brief WCID Compatible ID for DFU interface in DFU mode.
 */
//@{
__code USB_WCID_feature_descriptor wcid_feature_dfumode =
{
	sizeof(struct USB_WCID_feature_descriptor), /* dwLength */
	USB_MICROSOFT_WCID_VERSION, /* bcdVersion */
	USB_MICROSOFT_WCID_FEATURE_WINDEX_COMPAT_ID, /* wIndex */
	1, /* bCount */
	{0, 0, 0, 0, 0, 0, 0,}, /* rsv1 */
	DFU_USB_INTERFACE_DFUMODE, /* bFirstInterfaceNumber */
	1, /* rsv2 - set to 1 */
	{'W', 'I', 'N', 'U', 'S', 'B', 0x00, 0x00,}, /* compatibleID[8] */
	{0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,}, /* subCompatibleID[8] */
	{0, 0, 0, 0, 0, 0,}, /* rsv3[6] */
};
//@}

/**
 @name wcid_device_guid_dfu
 @brief WCID Compatible DeviceInterfaceGUIDs for DFU interface.
 */
//@{
__code USB_WCID_extended_property_descriptor wcid_device_guid_dfu =
{
	sizeof(struct USB_WCID_extended_property_descriptor), /* dwLength */
	USB_MICROSOFT_WCID_VERSION, /* bcdVersion */
	USB_MICROSOFT_WCID_FEATURE_WINDEX_DEVICE_GUID, /* wIndex */
	1, /* wCount */
	136, /* dwSize */
	USB_MICROSOFT_WCID_PROPERTY_DATATYPE_STRING_MULTI, /* dwPropertyDataType */
	42, /* wPropertyNameLength */
	{'D', 0x00, 'e', 0x00, 'v', 0x00, 'i', 0x00, 'c', 0x00, 'e', 0x00,
	 'I', 0x00, 'n', 0x00, 't', 0x00, 'e', 0x00, 'r', 0x00, 'f', 0x00,
	 'a', 0x00, 'c', 0x00, 'e', 0x00, 'G', 0x00, 'U', 0x00, 'I', 0x00,
	 'D', 0x00, 's', 0x00, 0x00, 0x00 }, /* null terminated PropertyName */
	80, /* dwPropertyDataLength */
	/* GUID for FT51A DFU interfaces "{1eaaaa95-1bd6-49c2-b4ae-286583f61227}" */
	{'{', 0x00, '1', 0x00, 'e', 0x00, 'a', 0x00, 'a', 0x00, 'a', 0x00,
	 'a', 0x00, '9', 0x00, '5', 0x00, '-', 0x00, '1', 0x00, 'b', 0x00,
	 'd', 0x00, '6', 0x00, '-', 0x00, '4', 0x00, '9', 0x00, 'c', 0x00,
	 '2', 0x00, '-', 0x00, 'b', 0x00, '4', 0x00, 'a', 0x00, 'e', 0x00,
	 '-', 0x00, '2', 0x00, '8', 0x00, '6', 0x00, '5', 0x00, '8', 0x00,
	 '3', 0x00, 'f', 0x00, '6', 0x00, '1', 0x00, '2', 0x00, '2', 0x00,
	 '7', 0x00, '}', 0x00,
	 0x00, 0x00, 0x00, 0x00}, /* bPropertyData */
};
//@}

/**
     @name LED definitions
     @brief Definitions for Digital IO port and bitmasks for LEDs.
         Two LEDs are defined, one for heartbeat and one for DFU
         in runtime mode.
         A millisecond counter is used to make flashing LED. The state
         of the LED will toggle when the timer expires.
 */
//@{
/// Digital Output port for LED timer.
#define LED_PORT P3

/// Bitmask for LED showing DFU active.
#define LED_DFU_ACTIVE_BITMASK 0x02
/// Bitmask for LED showing timer activity.
#define LED_FLASH_BITMASK 0x01
/// Countdown value for LED timer.
uint16_t led_count;
/// Toggle counter for LED timer.
uint16_t led_period;
/// Default period of LED flash
#define LED_FLASH_PERIOD_DEFAULT 500
/// Rapid period of LED flash
#define LED_FLASH_PERIOD_RAPID 50
//@}

/**
     @name Countdown timer
         A millisecond counter is used to detach the device when
         required by the DFU.
 */
static volatile uint16_t ms_timer = 0;

void usb_setup(void);
FT51_STATUS class_req_cb(USB_device_request *req);
FT51_STATUS standard_req_get_descriptor(USB_device_request *req);
FT51_STATUS standard_req_get_status(USB_device_request *req);
FT51_STATUS standard_req_get_set_feature(USB_device_request *req);
FT51_STATUS standard_req_cb(USB_device_request *req);
FT51_STATUS vendor_req_cb(USB_device_request *req);
void suspend_cb(uint8_t status);
void resume_cb(uint8_t status);
void reset_cb(uint8_t status);
#if DFU_DETACH_MODE
void usb_detach_attach(void);
#endif // DFU_DETACH_MODE

/**
    @brief      Invoked by ISR for Timer0 every millisecond
    @details    Each millisecond Timer0 interrupt handler will call
     	 this routine. When the detach counter is active (> zero) then
     	 we will decrement it. If we are in the DFUSTATE_APPDETACH state
     	 when it reaches zero then we will assume that the host has not
     	 send the USB reset required to mode to DFU Mode and revert
     	 the state machine back to DFUSTATE_APPIDLE to wait for another
     	 DFU_DETACH request.
    @param[in]  flags (not used).
 **/
void detach_interrupt(const uint8_t flags)
{
    (void)flags; // Flags not currently used

    // The DFU detach timer must be called once per millisecond
    if (dfu_timer())
    {
        led_period = LED_FLASH_PERIOD_DEFAULT;
    }

    // Toggle heartbeat LED when timer expires.
    if (led_count == 0)
    {
        led_count = led_period;
#ifdef ENABLE_LEDS
        LED_PORT ^= LED_FLASH_BITMASK;
#endif // ENABLE_LEDS
    }
    led_count--;

    if (ms_timer > 0)
    {
        ms_timer--;
    }

    // Reload the timer
    TH0 = MSB(MICROSECONDS_TO_TIMER_TICKS(1000));
    TL0 = LSB(MICROSECONDS_TO_TIMER_TICKS(1000));
}

/**
    @brief      Initialise Timer0 to fire every millisecond.
    @details    Associate the detach_interrupt handler function with
    		the Timer0 interrupt.
 **/
void detach_timer_initialise(void)
{
    // Register our own handler for interrupts from Timer 0
    interrupts_register(detach_interrupt, interrupts_timer0);

    // Timer0 is controlled by TMOD bits 0 to 3, and TCON bits 4 to 5.
    TMOD &= 0xF0; // Clear Timer0 bits
    TMOD |= 0x01; // Put Timer0 in mode 1 (16 bit)
    // Set the count-up value so that it rolls over to 0 after 1 millisecond.
    TH0 = MSB(MICROSECONDS_TO_TIMER_TICKS(1000));
    TL0 = LSB(MICROSECONDS_TO_TIMER_TICKS(1000));
    TCON &= 0xCF; // Clear Timer0's Overflow and Run flags
    TCON |= 0x10; // Start Timer0 (set its Run flag)
}

/**
    @brief      USB class request handler
    @details    Handle class requests from the host application.
    			The bRequest value is parsed and the appropriate
    			action or function is performed. Additional values
    			from the USB_device_request structure are decoded
    			and the DFU state machine and status updated as
    			required.
    @param[in]	req - USB_device_request structure containing the
    			SETUP portion of the request from the host.
    @return		status - FT51_OK if successful or FT51_FAILED
    			if there is an error or the bRequest is not
    			supported.
 **/
FT51_STATUS class_req_cb(USB_device_request *req)
{
    FT51_STATUS         status = FT51_FAILED;
    uint8_t             interface = LSB(req->wIndex) & 0x0F;

    // For DFU requests the SETUP packet must be addressed to the
    // the DFU interface on the device.

    // For DFU requests ensure the recipient is an interface...
    if ((req->bmRequestType & USB_BMREQUESTTYPE_RECIPIENT_MASK) ==
            USB_BMREQUESTTYPE_RECIPIENT_INTERFACE)
    {
        // ...and that the interface is the correct Runtime interface
        if (dfu_is_runtime())
        {
            if ((interface == DFU_USB_INTERFACE_RUNTIME))
            {
                // Handle only DFU_DETATCH, DFU_GETSTATE and DFU_GETSTATUS
                // when in Runtime mode. Table 3.2 DFU_DETACH is mandatory
                // in Runtime mode, DFU_GETSTATE and DFU_GETSTATUS are
                // optional.
                switch (req->bRequest)
                {
                case USB_CLASS_REQUEST_DETACH:
                    dfu_class_req_detach(req->wValue);
                    // Rapid LED flash
                    led_period = LED_FLASH_PERIOD_RAPID;
#if DFU_DETACH_MODE
                    usb_detach_attach();
#endif // DFU_DETACH_MODE
                    status = FT51_OK;
                    break;
                case USB_CLASS_REQUEST_GETSTATUS:
                    dfu_class_req_getstatus();
                    status = FT51_OK;
                    break;
                case USB_CLASS_REQUEST_GETSTATE:
                    dfu_class_req_getstate();
                    status = FT51_OK;
                    break;
                }
            }
        }
        // ...or the correct DFU Mode interface
        else
        {
            if (interface == DFU_USB_INTERFACE_DFUMODE)
            {
                // Handle remaining DFU class requests when in DFU Mode.
                // Table 3.2 DFU_DETACH is not supported in DFU Mode.
                switch (req->bRequest)
                {
                case USB_CLASS_REQUEST_DNLOAD:
                    /* For simplicity only program the maximum block size
                     * starting at an address which is an exact multiple
                     * of this. The block size is given in the configuration
                     * descriptor for the DFU interface.
                     */
                    // Rapid LED flash
                    led_period = LED_FLASH_PERIOD_RAPID;
                    dfu_class_req_download(req->wValue * DFU_BLOCK_SIZE, req->wLength);
                    status = FT51_OK;
                    break;
                case USB_CLASS_REQUEST_GETSTATUS:
                    dfu_class_req_getstatus();
#if DFU_DETACH_MODE
                    if (dfu_is_wait_reset())
                    {
                    	usb_detach_attach();
                    }
#endif // DFU_DETACH_MODE
                    status = FT51_OK;
                    break;
                case USB_CLASS_REQUEST_GETSTATE:
                    dfu_class_req_getstate();
                    status = FT51_OK;
                    break;
                case USB_CLASS_REQUEST_CLRSTATUS:
                    dfu_class_req_clrstatus();
                    status = FT51_OK;
                    break;
                case USB_CLASS_REQUEST_ABORT:
                    dfu_class_req_abort();
                    // Normal LED flash
                    led_period = LED_FLASH_PERIOD_DEFAULT;
                    status = FT51_OK;
                    break;

                default:
                case USB_CLASS_REQUEST_UPLOAD:
                    // Unknown or unsupported request.
                    break;
                }
            }
        }
    }

    return status;
}

/**
    @brief      USB standard request handler for GET_DESCRIPTOR
    @details    Handle standard GET_DESCRIPTOR requests from the host
    			application.
    			The hValue is parsed and the appropriate device,
    			configuration or string descriptor is selected.
    			For device and configuration descriptors the DFU
    			state machine determines which descriptor to use.
    			For string descriptors the lValue selects which
    			string from the string_descriptors table to use.
    			Strings are obtained from the __code section and
    			typecast to a be compatible with __xdata storage.
    			The string table is automatically traversed to find
    			the correct string and the length is taken from the
    			string descriptor.
    @param[in]	req - USB_device_request structure containing the
    			SETUP portion of the request from the host.
    @return		status - FT51_OK if successful or FT51_FAILED
    			if there is an error or the bmRequestType is not
    			supported.
 **/
FT51_STATUS standard_req_get_descriptor(USB_device_request *req)
{
    uint8_t   *src = NULL;
    uint16_t   length = req->wLength;
    uint8_t    hValue = req->wValue >> 8;
    uint8_t    lValue = req->wValue & 0x00ff;
    uint8_t    i;
    uint8_t    slen;

    switch (hValue)
    {
    case USB_DESCRIPTOR_TYPE_DEVICE:

        if (dfu_is_runtime())
        {
            src = (char *)&device_descriptor_runtime;
        }
        else
        {
            src = (char *)&device_descriptor_dfumode;
        }
        if (length > sizeof(USB_device_descriptor)) // too many bytes requested
            length = sizeof(USB_device_descriptor); // Entire structure.
        break;

    case USB_DESCRIPTOR_TYPE_CONFIGURATION:

        if (dfu_is_runtime())
        {
            src = (char *)&config_descriptor_runtime;
            if (length > sizeof(config_descriptor_runtime)) // too many bytes requested
                length = sizeof(config_descriptor_runtime); // Entire structure.
        }
        else
        {
            src = (char *)&config_descriptor_dfumode;
            if (length > sizeof(config_descriptor_dfumode)) // too many bytes requested
                length = sizeof(config_descriptor_dfumode); // Entire structure.
        }
        break;

    case USB_DESCRIPTOR_TYPE_STRING:

		// Special case: WCID descriptor
		if (lValue == USB_MICROSOFT_WCID_STRING_DESCRIPTOR)
		{
			src = (uint8_t *)wcid_string_descriptor;
			length = sizeof(wcid_string_descriptor);
			break;
		}

		// Find the nth string in the string descriptor table
        i = 0;
        while ((slen = string_descriptor[i]) > 0)
        {
            // Point to start of string descriptor in __code segment
            // Typecast prevents warning for losing const when usb_transfer
            // is called
            src = (uint8_t *)&string_descriptor[i];
            if (lValue == 0)
            {
                break;
            }
            i += slen;
            lValue--;
        }
        if (lValue > 0)
        {
            // String not found
            return FT51_FAILED;
        }
        // Update the length returned only if it is less than the requested
        // size
        if (length > slen)
        {
            length = slen;
        }

        break;

    default:
        return FT51_FAILED;
    }

    USB_transfer(USB_EP_0, USB_DIR_IN, src, length);

    return FT51_OK;
}

/**
    @brief      USB standard request handler for GET_STATUS
    @details    Handle standard GET_STATUS requests from the host
    			application.
    			The bmRequestType value is parsed and the appropriate
    			action or function is performed.
    @param[in]	req - USB_device_request structure containing the
    			SETUP portion of the request from the host.
    @return		status - FT51_OK if successful or FT51_FAILED
    			if there is an error or the bmRequestType is not
    			supported.
 **/
FT51_STATUS standard_req_get_status(USB_device_request *req)
{
    USB_STATE state = USB_get_state();
    USB_ENDPOINT_NUMBER ep_number = LSB(req->wIndex) & 0x0F;
    USB_ENDPOINT_DIR ep_dir = (LSB(req->wIndex) >> 7);
    uint8_t buf[2];

    buf[0] = buf[1] = 0;

    // Get Status for endpoints only
    if (req->bmRequestType == (USB_BMREQUESTTYPE_DIR_DEV_TO_HOST |
            USB_BMREQUESTTYPE_RECIPIENT_ENDPOINT))
    {
        if (((state < CONFIGURED) && (ep_number == 0))
                || (state >= CONFIGURED))
        {
            if (USB_get_ep_stalled(ep_number, ep_dir))
            {
                buf[0] = USB_GET_STATUS_ENDPOINT_HALT;
            }
            USB_transfer(USB_EP_0, USB_DIR_IN, buf, 2);

            return FT51_OK;
        }
    }
    else if (req->bmRequestType == (USB_BMREQUESTTYPE_DIR_DEV_TO_HOST |
            USB_BMREQUESTTYPE_RECIPIENT_DEVICE))
    {
        // This must match the configuration descriptor's bmAttributes
        buf[0] = USB_CONFIG_BMATTRIBUTES_VALUE;
#if USB_REMOTE_WAKEUP == 1
        if (USB_get_remote_wakeup())
        {
            buf[0] |= USB_GET_STATUS_DEVICE_REMOTE_WAKEUP;
        }
#endif // USB_REMOTE_WAKEUP
        
        USB_transfer(USB_EP_0, USB_DIR_IN, buf, 2);

        return FT51_OK;
    }
    return FT51_FAILED;
}

/**
 @brief      USB standard request handler for GET_FEATURE and SET_FEATURE
 @details    Handle standard GET_FEATURE and SET_FEATURE requests
             from the host application.
             The bmRequestType value is parsed and the appropriate
             action or function is performed.
 @param[in]  req - USB_device_request structure containing the
             SETUP portion of the request from the host.
 @return        status - FT51_OK if successful or FT51_FAILED
             if there is an error or the bmRequestType is not
             supported.
 **/
FT51_STATUS standard_req_set_clear_feature(USB_device_request *req)
{
    USB_ENDPOINT_NUMBER ep_number = LSB(req->wIndex) & 0x0F;
    USB_ENDPOINT_DIR ep_dir = (LSB(req->wIndex) >> 7);
    USB_STATE state = USB_get_state();

    if (req->bmRequestType == (USB_BMREQUESTTYPE_DIR_HOST_TO_DEV |
            USB_BMREQUESTTYPE_RECIPIENT_ENDPOINT))
    {
        // Support the endpoint halt feature in this device
        if (req->wValue == USB_FEATURE_ENDPOINT_HALT)
        {
            // Only allow this if the device is configured or the endpoint
            // is zero if the device is not configured.
            if (((state < CONFIGURED) && (ep_number == 0))
                    || (state >= CONFIGURED))
            {
                // Perform a stall or a clear
                if (req->bRequest == USB_REQUEST_CODE_CLEAR_FEATURE)
                {
                    // Feature selector CLEAR
                    USB_clear_endpoint(ep_number, ep_dir);
                }
                else
                {
                    // Feature selector SET
                    USB_stall_endpoint(ep_number, ep_dir);
                }
                USB_transfer(USB_EP_0, USB_DIR_IN, NULL, 0);
                return FT51_OK;
            }
        } 
    }
    if (req->bmRequestType == (USB_BMREQUESTTYPE_DIR_HOST_TO_DEV |
            USB_BMREQUESTTYPE_RECIPIENT_DEVICE))
    {
#if USB_REMOTE_WAKEUP == 1
        // Only support the remote wakeup feature in this device
        if (req->wValue == USB_FEATURE_DEVICE_REMOTE_WAKEUP)
        {
            // Perform a stall or a clear
            if (req->bRequest == USB_REQUEST_CODE_CLEAR_FEATURE)
            {
                // Feature selector CLEAR
                USB_clear_remote_wakeup();
            }
            else
            {
                // Feature selector SET
                USB_set_remote_wakeup();
            }

            USB_transfer(USB_EP_0, USB_DIR_IN, NULL, 0);
            return FT51_OK;
        }
#endif // USB_REMOTE_WAKEUP
    }
    
    return FT51_FAILED;
}

/**
    @brief      USB standard request handler
    @details    Handle standard requests from the host application.
    			The bRequest value is parsed and the appropriate
    			action or function is performed. Additional values
    			from the USB_device_request structure are decoded
				and provided to other handlers.
    @param[in]	req - USB_device_request structure containing the
    			SETUP portion of the request from the host.
    @return		status - FT51_OK if successful or FT51_FAILED
    			if there is an error or the bRequest is not
    			supported.
 **/
FT51_STATUS standard_req_cb(USB_device_request *req)
{
    FT51_STATUS status = FT51_FAILED;

    switch (req->bRequest)
    {
    case USB_REQUEST_CODE_GET_STATUS:
        status = standard_req_get_status(req);
        break;

    case USB_REQUEST_CODE_CLEAR_FEATURE:
    case USB_REQUEST_CODE_SET_FEATURE:
        status = standard_req_set_clear_feature(req);
        break;

    case USB_REQUEST_CODE_GET_DESCRIPTOR:
        status = standard_req_get_descriptor(req);
        break;

    case USB_REQUEST_CODE_SET_CONFIGURATION:
        status = USB_req_set_configuration(req);

        if (status == FT51_OK)
        {
            /* Make application specific BULK, INTERRUPT or
             * ISOCHRONOUS endpoints.
             */
        }
        break;

    case USB_REQUEST_CODE_GET_CONFIGURATION:
        status = USB_req_get_configuration();
        break;

    case USB_REQUEST_CODE_SET_ADDRESS:
        status =  USB_req_set_address(req);
        break;

    default:
    case USB_REQUEST_CODE_GET_INTERFACE:
    case USB_REQUEST_CODE_SET_INTERFACE:
        // Unknown or unsupported request.
        status = FT51_FAILED;
        break;
    }

    return status;
}

/**
    @brief      USB vendor request handler
    @details    Handle vendor requests from the host application.
    			The bRequest value is parsed and the appropriate
    			action or function is performed. Additional values
    			from the USB_device_request structure are decoded
				and provided to other handlers.
				There are no vendor requests requiring handling in
				this firmware.
    @param[in]	req - USB_device_request structure containing the
    			SETUP portion of the request from the host.
    @return		status - FT51_OK if successful or FT51_FAILED
    			if there is an error or the bRequest is not
    			supported.
 **/
FT51_STATUS vendor_req_cb(USB_device_request *req)
{
    FT51_STATUS status = FT51_FAILED;
	uint16_t length = req->wLength;

	// For Microsoft WCID only.
	// Request for Compatible ID Feature Descriptors.
	// Check the request code and wIndex.
	if (req->bRequest == WCID_VENDOR_REQUEST_CODE)
	{
		if (req->bmRequestType & USB_BMREQUESTTYPE_DIR_DEV_TO_HOST)
		{
			switch (req->wIndex)
			{
			case USB_MICROSOFT_WCID_FEATURE_WINDEX_COMPAT_ID:
				if (length > sizeof(wcid_feature_runtime)) // too many bytes requested
					length = sizeof(wcid_feature_runtime); // Entire structure.
				// Return a compatible ID feature descriptor.
				if (dfu_is_runtime())
				{
					USB_transfer(USB_EP_0, USB_DIR_IN,
						(uint8_t *) &wcid_feature_runtime, length);
					status = FT51_OK;
				}
				else
				{
					USB_transfer(USB_EP_0, USB_DIR_IN,
						(uint8_t *) &wcid_feature_dfumode, length);
					status = FT51_OK;
				}
				break;

			case USB_MICROSOFT_WCID_FEATURE_WINDEX_DEVICE_GUID:
				if (length > sizeof(wcid_device_guid_dfu)) // too many bytes requested
					length = sizeof(wcid_device_guid_dfu); // Entire structure.
				// Return a compatible ID feature descriptor.
				if (((dfu_is_runtime()) && (req->wValue == DFU_USB_INTERFACE_RUNTIME))
					|| ((!dfu_is_runtime()) && (req->wValue == DFU_USB_INTERFACE_DFUMODE)))
				{
					USB_transfer(USB_EP_0, USB_DIR_IN,
							(uint8_t *) &wcid_device_guid_dfu, length);
					status = FT51_OK;
				}
				break;
			}
		}
	}

	return status;
}

/**
    @brief      USB suspend callback
    @details    Called when the USB bus enters the suspend state.
    @param[in]  status - unused.
    @return     N/A
 **/
void suspend_cb(uint8_t status)
{
    (void)status;
    led_period = LED_FLASH_PERIOD_DEFAULT * 2;
    return;
}

/**
    @brief      USB resume callback
    @details    Called when the USB bus enters the resume state
                prior to restating after a suspend.
    @param[in]  status - unused.
    @return     N/S
 **/
void resume_cb(uint8_t status)
{
    (void)status;
    led_period = LED_FLASH_PERIOD_DEFAULT;
    return;
}

/**
    @brief      USB reset callback
    @details    Called when the USB bus enters the reset state.
    @param[in]  status - unused.
    @return     N/A
 **/
void reset_cb(uint8_t status)
{
    (void) status;

    USB_set_state(DEFAULT);

    if (dfu_reset())
    {
        // Run downloaded application - reload MTP to shadow RAM
        // Execution effectively stops now and returns to the main()
        // function of the new program in MTP.
        device_revert()
        ;
    }

#ifdef ENABLE_LEDS
    if (dfu_is_runtime())
    {
        // Runtime mode - DFU active LED off
        LED_PORT |= LED_DFU_ACTIVE_BITMASK;
    }
    else
    {
        // DFU mode - DFU active LED on
        LED_PORT &= (~LED_DFU_ACTIVE_BITMASK);
    }
#endif // ENABLE_LEDS

    return;
}

/**
 @brief      USB detach attach sequence
 @details    Called when the USB device wishes to detach from the
 	 	 	 bus then reattach.
 @return		N/A
 **/
#if DFU_DETACH_MODE
void usb_detach_attach(void)
{
	ms_timer = 10;
	while (ms_timer > 0);
	// Disable USB device to force an reset
	USB_finalise();
	ms_timer = 10;
	while (ms_timer > 0);
	// Start the USB device.
	usb_setup();
}
#endif // DFU_DETACH_MODE

/**
    @brief      Initialise emulation of a USB device
    @details    Sets up callback functions and data structures
                required to implement control endpoints on device.
    @param[in]  ep_size - Endpoint size of control endpoints.
    @return     N/A
 **/
void usb_setup(void)
{
    USB_ctx usb_ctx;

    memset(&usb_ctx, 0, sizeof(usb_ctx));

    usb_ctx.standard_req = standard_req_cb;
    usb_ctx.class_req = class_req_cb;
    usb_ctx.vendor_req = vendor_req_cb;
    usb_ctx.suspend = suspend_cb;
    usb_ctx.resume = resume_cb;
    usb_ctx.reset = reset_cb;
    usb_ctx.lpm = NULL;

    usb_ctx.hub_en.mask.hub_enable = USB_HUB_ENABLED;
    usb_ctx.hub_en.mask.hub_LocalPwrSrc = USB_SELF_POWERED;
    usb_ctx.hub_en.mask.hub_RmtWkUpEn = 1;

    // Initialise the USB device with a control endpoint size
    // of 8 bytes. This must match the size for bMaxPacketSize0
    // defined in the device descriptor.
    usb_ctx.ep_size = USB_CONTROL_EP_SIZE;

    // Perform USB Device initialisation.
    USB_initialise(&usb_ctx);

    return;
}

/**
 @brief      Initialise I/O pins of FT51A
 @details    Sets up IOMUX to route signals to I/O pins.
 **/
void iomux(void)
{
#if USB_REMOTE_WAKEUP == 1
    IOMUX_INPUT (PIN_REMOTE_WAKEUP, IOMUX_IN_MISC_WAKEUP_PIN);
    IOMUX_set_pad_config(PIN_REMOTE_WAKEUP,
                        IOMUX_PAD_DRIVE_CURRENT_4MA,
                        IOMUX_PAD_TRIGGER_NORMAL,
                        IOMUX_PAD_SLEW_RATE_FAST,
                        IOMUX_PAD_PULL_UP);
#endif // USB_REMOTE_WAKEUP == 1
 
#ifdef ENABLE_LEDS
    IOMUX_OUTPUT(AIO_11, IOMUX_OUT_PORT3_0); /* Marked as Heartbeat LED */
    IOMUX_OUTPUT(DIO_1, IOMUX_OUT_PORT3_1); /* Marked as Force LED */
#endif // ENABLE_LEDS
}

/**
    @brief      main function.
    @details    Modify any configuration parameters which are updatable
     	 	 	and call initialisation routines.
     	 	 	Finally process USB requests as they arrive (for ever).
 **/
void main(void)
{
    led_count = 0; led_period = LED_FLASH_PERIOD_DEFAULT;

    device_descriptor_runtime.idVendor = device_config.idVendor;
    device_descriptor_runtime.idProduct = device_config.idProductRunTime;
    device_descriptor_runtime.iManufacturer = device_config.iManufacturer;
    device_descriptor_runtime.iSerialNumber = device_config.iSerialNumber;
    device_descriptor_runtime.iProduct = device_config.iProductRunTime;

    config_descriptor_runtime.interface.iInterface = device_config.iInterface;

    device_descriptor_dfumode.idVendor = device_config.idVendor;
    device_descriptor_dfumode.idProduct = device_config.idProductDFUMode;
    device_descriptor_dfumode.iManufacturer = device_config.iManufacturer;
    device_descriptor_dfumode.iSerialNumber = device_config.iSerialNumber;
    device_descriptor_dfumode.iProduct = device_config.iProductDFUMode;

    config_descriptor_dfumode.interface.iInterface = device_config.iInterface;

    // Initialise FT51
    device_initialise();

    // Setup IOMUX to route signals to pins
    iomux();

    // Initialise USB function
    usb_setup();

    // Initialise the timer to provide a millisecond countdown timer
    detach_timer_initialise();

    // Endless loop.
    while (1)
    {
        // Check control endpoints and handle SETUPs
        USB_process();
    }
}
